@@ -241,7 +241,7 @@ class Algorithms:
                 self._vprint(f"{_elapsed(t0)} - applying LSC with max gains {gains}: range = {self._minmax(frame)}")
         return frame
 
-    def wb(self, frame: np.ndarray, gains: Sized) -> np.ndarray:
+    def wb(self, frame: np.ndarray, gains: Sized, bayer_pattern: str = None) -> np.ndarray:
         """
         Multiply the RGB channels of the given frame by the given white balance
         coefficients. If there are only two coefficients instead of three, they
@@ -250,11 +250,24 @@ class Algorithms:
         """
         if gains is not None:
             t0 = time.time()
-            gains = np.array(gains)
-            gains = np.insert(gains, 1, 1.0) if len(gains) == 2 else gains
-            frame = frame * gains
+            wb = np.asarray(gains)
+            wb = np.insert(wb, 1, 1.0) if len(wb) == 2 else wb
+            if frame.ndim == 3:  # RGB mode
+                mode = "RGB"
+                frame = frame * wb
+            if frame.ndim == 2:  # Bayer mode
+                assert bayer_pattern is not None, "wb() requires bayer_pattern for raw Bayer frames"
+                assert bayer_pattern in ["RGGB", "BGGR", "GRBG", "GBRG"], bayer_pattern
+                mode = bayer_pattern
+                order = ["RGB".index(ch) for ch in bayer_pattern.upper()]
+                wb = wb[order]  # BGGR => [2, 1, 1, 0]
+                wb = wb.reshape(4, 1, 1)
+                channels = self.bayer_split(frame)
+                channels = channels * wb
+                frame = self.bayer_combine(*channels)
             with np.printoptions(formatter={'float': lambda x: f"{x:.3f}"}):
-                self._vprint(f"{_elapsed(t0)} - applying WB gains {gains}: range = {self._minmax(frame)}")
+                wb = wb.flatten()
+                self._vprint(f"{_elapsed(t0)} - applying WB gains {wb} in {mode} order: range = {self._minmax(frame)}")
         return frame
 
     def ccm(self, frame: np.ndarray, matrix: np.ndarray, clip=True) -> np.ndarray:
@@ -489,17 +502,17 @@ def _aces_gamut(rgb, threshold=np.array([0.815, 0.803, 0.880]), cyan=0.147, mage
 
     # thr is the percentage of the core gamut to protect.
     thr = np.clip(threshold, -np.inf, 0.9999)
-    thr = thr.reshape([1] * (rgb.ndim - 1) + [3])
+    thr = thr.reshape([1] * (rgb.ndim - 1) + [3])  # (1, 1, 3)
 
     # lim is the max distance from the gamut boundary that will be compressed
     # 0 is a no-op, 1 will compress colors from a distance of 2 from achromatic to the gamut boundary
     lim = np.array([cyan + 1, magenta + 1, yellow + 1])
 
     # achromatic axis
-    ach = np.max(rgb, axis=-1, keepdims=True)
+    ach = np.max(rgb, axis=-1, keepdims=True)  # always >= 0
 
     # distance from the achromatic axis for each color component aka inverse rgb ratios
-    dist = np.where(ach == 0.0, 0.0, (ach - rgb) / np.abs(ach))
+    dist = np.where(ach == 0.0, 0.0, (ach - rgb) / np.abs(ach))  # always >= 0
 
     # compress distance with user controlled parameterized shaper function
     cdist = compress(dist, lim, thr, power)
